#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "Randomize.hh"

#include "DetectorConstruction.hh"
#include "G4PhysListFactory.hh"
#include "G4VModularPhysicsList.hh"
#include "PrimaryGeneratorAction.hh"

#include "RunAction.hh"
#include "EventAction.hh"

#ifdef G4VIS_USE
#include "G4VisExecutive.hh"
#endif

#ifdef G4UI_USE
#include "G4UIExecutive.hh"
#endif

#include <sstream>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

int main(int argc,char** argv){
  //choose the Random engine
  CLHEP::HepRandom::setTheEngine(new CLHEP::RanecuEngine());

  //Construct the default run manager
  G4RunManager * runManager = new G4RunManager();

  //set mandatory initialization classes
  runManager->SetUserInitialization(new DetectorConstruction());

  G4PhysListFactory factory;
  G4VModularPhysicsList* phys = 0;

  // Physics List name defined via 2nd argument
  G4String physName = "LHEP";
  if (argc==3) {
    if(factory.IsReferencePhysList(argv[2])) physName = argv[2];
  }
  phys = factory.GetReferencePhysList(physName);

  // Physics List is defined via environment variable PHYSLIST
  if(!phys) phys = factory.ReferencePhysList();
  runManager->SetUserInitialization(phys);

  runManager -> Initialize();

  runManager->SetUserAction(new PrimaryGeneratorAction());

  //set user action classes
  runManager->SetUserAction(new RunAction());
  runManager->SetUserAction(new EventAction());

  //get the pointer to the User Interface manager
  G4UImanager* UImanager = G4UImanager::GetUIpointer();
  G4VisManager* visManager = 0;

#ifdef G4VIS_USE
  //visualization manager
  visManager = new G4VisExecutive;
  visManager->Initialize();
#endif

  if(argc==1)
    {
      UImanager->ApplyCommand("/gun/energy 230. MeV");
      for(G4double erel = 0.05; erel < 1.001*MeV; erel += 0.05*MeV){
	std::ostringstream oss;
	oss << "/gun/erel " << erel << " MeV";
	UImanager->ApplyCommand(oss.str());
	UImanager->ApplyCommand("/run/beamOn 40000");
      }
    }
  else           // Batch mode
    {
     G4String command = "/control/execute ";
     G4String fileName = argv[1];
     UImanager->ApplyCommand(command+fileName);
    }

  //job termination
#ifdef G4VIS_USE
  delete visManager;
#endif

  delete runManager;

  return 0;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
