//
#include "PrimaryGeneratorAction.hh"
#include "BeamMessenger.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "G4ParticleGun.hh"
#include "G4ThreeVector.hh"

#include "Randomize.hh"

#include "HitStatus.hh"

#include "CParticle.hh"
#include "CKinema.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PrimaryGeneratorAction::PrimaryGeneratorAction()
{
  beamMessenger = new BeamMessenger(this);
  particleGun = new G4ParticleGun(1);

  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  G4ParticleDefinition* particle = particleTable->FindParticle("geantino");
  particleGun->SetParticleDefinition(particle);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PrimaryGeneratorAction::~PrimaryGeneratorAction()
{
  delete particleGun;
  delete beamMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
  G4double energy(particleGun->GetParticleEnergy());
  G4ThreeVector momentum = CalculateMomentum();
  G4ThreeVector position = G4ThreeVector(0., 0., 0.);

  particleGun->SetParticleMomentumDirection(momentum);
  particleGun->SetParticlePosition(position);
  particleGun->SetParticleEnergy(energy);

  particleGun->GeneratePrimaryVertex(anEvent);

  HitStatus* hitStatus = HitStatus::GetHitStatus();
  hitStatus->SetEnergy(energy);
}

G4ThreeVector PrimaryGeneratorAction::CalculateMomentum()
{
  G4ThreeVector momentum;

  if(1){
    //two body
    HitStatus* hitStatus = HitStatus::GetHitStatus();
    hitStatus->SetErel(erel);

    int a1(1), a2(9);
    double p1_me(8.0713171*MeV), p2_me(24.954264*MeV);
    double u(931.494*MeV);
    CParticle p1(p1_me+u*a1),p2(p2_me+u*a2);

    G4double kin(particleGun->GetParticleEnergy());

    p1.k(0);
    p2.k(0);

    CKinema kinema;
    kinema.FissionErel(p1, p2, erel, kin);

    momentum = G4ThreeVector(p1.p(0), p1.p(1), p1.p(2));
  }else if(0){
    //sphere
    CLHEP::MTwistEngine MTwistEngine;
    G4double theta(2.0*M_PI*CLHEP::RandFlat::shoot(&MTwistEngine)*radian);
    G4double radius(pow(fabs(CLHEP::RandGauss::shoot(&MTwistEngine, 0.0, 1.0)), 1.0/2.0));

    momentum = G4ThreeVector(0.2*radius*cos(theta), 0.2*radius*sin(theta), 7.0);
  }else{
    //no spread
    momentum = G4ThreeVector(0., 0., 1.);
  }

  return momentum;
}

void PrimaryGeneratorAction::SetErel(G4double val)
{
  if(val >= 0.0) {
    erel = val;
  } 
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
